/*
 * z2-Environment
 * 
 * Copyright(c) ZFabrik Software GmbH & Co. KG
 * 
 * contact@zfabrik.de
 * 
 * http://www.z2-environment.eu
 */
import java.io.File;
import java.io.FileInputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Properties;


public class CommandLineComputer {
	private static final String CONFIG_FILE = "launch.properties";
	private static final String EMODE = "-Emode:";
	private static final String MODE = "-mode:";
	private static final String JAVAW = "-javaw";
	private final static String VMOPTS = "home.vmopts";
	private static List<String> 	vmopts = new ArrayList<String>();
	private static List<String> 	progopts =  new ArrayList<String>();
	private static Map<String,String> singleVal = new HashMap<String, String>();
	
	private static String computedClassPath;
	private static boolean useJavaw;
	
	/**
	 * This class computes a java commandline for a home process.
	 * It does so based on the actual command line and whatever
	 * defaults are defined in the runtime configuration.
	 * At first we evaluate the runtime properties as far as they
	 * define vm properties. At last we check the command line.
	 */
	public static void main(String[] args) throws Exception {
		hello();
		
		// create log output folder
		new File("../logs").mkdir();
		
		Properties rp = new Properties();
		rp.load(new FileInputStream(new File(CONFIG_FILE)));
		
		processOptsLine(rp.getProperty(VMOPTS));
		// check if there is modes 
		List<String> l = new ArrayList<String>(args.length);
		String a;
		boolean vmprops = true;
		for (int i=0;i<args.length;i++) {
			a = args[i];
			if (vmprops) {
				if (JAVAW.equals(a)) {
					useJavaw=true;
				} else	if (a.startsWith(EMODE)) {
					processOptsLine(rp.getProperty(VMOPTS+"."+a.substring(EMODE.length())));
				} else	if (a.startsWith(MODE)) {
					processOptsLine(rp.getProperty(VMOPTS+"."+a.substring(MODE.length())));
				} else if ("-".equals(a)) {
					vmprops = false;
				} else {
					l.add(args[i]);
				}
			} else	{
				progopts.add(a);
			}
		}
		// process the command line props
		processOpts(l.toArray(new String[l.size()]));
		StringBuffer cmdline;
		if (useJavaw) {
			cmdline = new StringBuffer("javaw");
		} else {
			cmdline = new StringBuffer("java");
		}
		for (String b : vmopts) {
			cmdline.append(" ");
			cmdline.append(b);
		}
		cmdline.append(" launchers.HomeLauncher");
		for (String b : progopts) {
			cmdline.append(" ");
			cmdline.append(b);
		}
		System.out.print(cmdline.toString());		
	}


//	private static void usage() {
//		try {
//			BufferedReader in = new BufferedReader(new InputStreamReader(
//					CommandLineComputer.class.getResourceAsStream("launchers/usage.txt")));
//			String line;
//			while ((line = in.readLine()) != null) {
//				System.err.println(line);
//			}
//		} catch (IOException e) {
//			e.printStackTrace();
//		}
//		System.exit(1);
//	}



	private static void hello() {
	}


	private static void processOptsLine(String property) {
		if (property==null) return;
		processOpts(property.split(" "));
	}

	//
	// unfortunately vmoptions are not well defined grammared.
	// for some, we check them as prefix so that we can overwrite 
	// them cleanly.
	//
	private static void processOpts(String[] strings) {
		String o;
		boolean cp = false;
		for (String a : strings) {
			if (cp) {
				a = (computedClassPath==null? "" : computedClassPath+File.pathSeparatorChar)+a;
				cp = false;
				// put new value
				singleVal.put("-cp",a);
			} else {
				if (a.startsWith("-cp")) {
					// classpath needs fixing based on computed classpath
					// note that the next one is the cp.
					cp = true;
					// remove previous clp
					o = singleVal.get("-cp");
					if (o!=null) {
						vmopts.remove(o);
						vmopts.remove("-cp");
					}
				} else {
					// identify the key of that command line option. (checking for an = if present)
					String key = a;
					int p = a.indexOf('=');
					if (p>=0) {
						key = a.substring(0,p);
					}
					o = singleVal.get(key);
					if (o!=null)
						vmopts.remove(o);
					// put new value
					singleVal.put(key,a);
				}
			}
			vmopts.add(a);
		}
	}

}
