/*
 * z2-Environment
 * 
 * Copyright(c) ZFabrik Software GmbH & Co. KG
 * 
 * contact@zfabrik.de
 * 
 * http://www.z2-environment.eu
 */
package com.zfabrik.impl.components.states;

import static com.zfabrik.impl.components.ComponentDependencyUtil.addDepsFromCSL;
import static com.zfabrik.impl.components.ComponentDependencyUtil.prepareDependencies;

import java.util.Collection;
import java.util.HashSet;
import java.util.Set;
import java.util.logging.Logger;

import com.zfabrik.components.IComponentDescriptor;
import com.zfabrik.components.IComponentsManager;
import com.zfabrik.components.IDependencyComponent;
import com.zfabrik.impl.components.ComponentDependencyUtil.IDependencyResolver;
import com.zfabrik.resources.IResourceHandle;
import com.zfabrik.resources.ResourceBusyException;
import com.zfabrik.resources.provider.Resource;
import com.zfabrik.states.ISystemState;
import com.zfabrik.util.expression.X;

/**
 * See {@link ISystemState}.
 * @author hb
 *
 */
public class SystemStateResource extends Resource implements IDependencyComponent, Runnable, ISystemState {
	private boolean attained;
	private final String name;
	private final X findStateParts;
	
	
	public SystemStateResource(String name) {
		this.name = name;
		this.findStateParts = X.val(this.name).in(X.var(STATES_PARTICIPATION));
	}
	
	public synchronized <T> T as(Class<T> clz) {
		if (IDependencyComponent.class.equals(clz)) {
			return clz.cast(this);
		}
		if (Runnable.class.equals(clz)) {
			return clz.cast(this);
		}
		if (ISystemState.class.equals(clz)) {
			return clz.cast(this);
		} 
		return null;
	}

	@Override
	public void run() {
		this.prepare();
	}
	
	@Override
	public void prepare() {
		_prepareDependencies();
		if (!this.attained) {
			handle().adjust(0, Long.MAX_VALUE, IResourceHandle.HARD);
			this.attained=true;
			logger.info("Attained system state: "+this.name);
		}
	}
	
	@Override
	public void invalidate() throws ResourceBusyException {
		synchronized (this) {
			if (this.attained) {
				logger.info("Left system state: "+this.name);
			}
			this.attained = false;
		}
	}
	
	private void _prepareDependencies() {
		prepareDependencies(
			handle(),
			new IDependencyResolver() {
				public Collection<String> getDependencies() throws Exception {
					Set<String> deps = new HashSet<String>();
					addDepsFromCSL(deps,handle().as(IComponentDescriptor.class).getProperty(STATES_DEPENDENCY));
					deps.addAll(IComponentsManager.INSTANCE.findComponents(findStateParts));
					return deps;
				}
			},
			logger
		);
	}
	
	private final static Logger logger = Logger.getLogger(SystemStateResource.class.getName()); 
}
