/*
 * z2-Environment
 * 
 * Copyright(c) ZFabrik Software GmbH & Co. KG
 * 
 * contact@zfabrik.de
 * 
 * http://www.z2-environment.eu
 */
package com.zfabrik.impl.dev.transport.gui;

import java.awt.Component;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.DefaultComboBoxModel;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JPasswordField;
import javax.swing.JTextField;
import javax.swing.SwingConstants;

import com.zfabrik.components.IComponentDescriptor;
import com.zfabrik.components.IComponentsLookup;

public class DestinationChooser extends JPanel implements ActionListener {
	private static final long serialVersionUID = 1L;
	private Destination svnDestination = new Destination();
	private String[] dests;
	
	private JComboBox sw_list;
	private JTextField sw_url, sw_user;
	private JPasswordField sw_passwd;
	
//	private ActionListener parent;

	private String currentSel;
	
	public Destination getDestination() {
		this.svnDestination.setPasswd(new String(this.sw_passwd.getPassword()));
		this.svnDestination.setUser(this.sw_user.getText());
		this.svnDestination.setUrl(this.sw_url.getText());
		this.svnDestination.setName(this.sw_url.getText());
		return this.svnDestination;
	}
	
	public String getUserName() {
	    return this.sw_user.getText();
	}
	
	public DestinationChooser(ActionListener parent, String[] dests) {
//		this.parent = parent;
		GridBagLayout grid = new GridBagLayout();
		GridBagConstraints c = new GridBagConstraints();
		c.fill = GridBagConstraints.BOTH;
		c.insets = new Insets(0,10,2,0);
		setLayout(grid);

		// combo box
		this.sw_list = new JComboBox();
		setDestinations(dests);
		this.sw_list.addActionListener(this);

		c.gridwidth=GridBagConstraints.REMAINDER;
		c.weightx = 1.0;
		_addWithConstraints(grid,c,this.sw_list);

		c.gridwidth=1;
		c.weightx = 0;
		_addWithConstraints(grid,c,_makeLabel("Url"));
		c.gridwidth=GridBagConstraints.REMAINDER;
		c.weightx = 1;
		_addWithConstraints(grid,c,(this.sw_url=new JTextField(30)));
		
		c.gridwidth=1;
		c.weightx = 0;
		_addWithConstraints(grid,c,_makeLabel("User"));
		c.gridwidth=GridBagConstraints.REMAINDER;
		c.weightx = 1;
		_addWithConstraints(grid,c,(this.sw_user=new JTextField(20)));

		c.gridwidth=1;
		c.weightx = 0;
		_addWithConstraints(grid,c,_makeLabel("Password"));
		c.gridwidth=GridBagConstraints.REMAINDER;
		c.weightx = 1;
		_addWithConstraints(grid,c,(this.sw_passwd=new JPasswordField(20)));

	}

	private void _addWithConstraints(GridBagLayout grid,
			GridBagConstraints c, Component comp) {
		grid.setConstraints(comp, c);
		add(comp);
	}

	private JLabel _makeLabel(String title) {
		JLabel l=new JLabel(title);
		l.setHorizontalAlignment(SwingConstants.RIGHT);
		return l;
	}
	
	public void setDestinations(String[] dests) {
		this.dests = dests;
		this.sw_list.setModel(new DefaultComboBoxModel(this.dests));
		if (this.dests.length>0) {
			this.sw_list.setSelectedIndex(0);
		}
	}
	
	
	public void actionPerformed(ActionEvent e) {
		Object s = e.getSource();
		if (s==this.sw_list) {
			String d = (String) ((JComboBox)s).getSelectedItem();
			if (!d.equals(this.currentSel)) {
				this.currentSel = d;
				IComponentDescriptor de = IComponentsLookup.INSTANCE.lookup(d, IComponentDescriptor.class);
				if (de==null) {
					TransportTool.error(this,"No destination found at \""+d+"\". Please refresh!",null);
					this.sw_url.setText("");
					this.sw_user.setText("");
					this.sw_passwd.setText("");
				} else {
					this.sw_url.setText(de.getProperties().getProperty(Destination.SVNCR_URL));
					this.sw_user.setText(de.getProperties().getProperty(Destination.SVNCR_USER));
					this.sw_passwd.setText(de.getProperties().getProperty(Destination.SVNCR_PWD));
				}
			}
		}
	}


//	private final static Logger logger = Logger.getLogger(DestinationChooser.class.getName());
}
