/*
 * z2-Environment
 * 
 * Copyright(c) ZFabrik Software GmbH & Co. KG
 * 
 * contact@zfabrik.de
 * 
 * http://www.z2-environment.eu
 */
package com.zfabrik.impl.dev.transport.gui;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.Calendar;
import java.util.Date;
import java.util.logging.Handler;
import java.util.logging.Level;
import java.util.logging.LogRecord;
import java.util.logging.Logger;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.KeyStroke;
import javax.swing.SwingUtilities;
import javax.swing.text.BadLocationException;

import com.zfabrik.work.ApplicationThreadPool;

public class ProgressLogger {
	private JDialog di;
	private JTextArea ta;
	private Logger l;
	private JButton bu,bc;
	private boolean done;
	private boolean textOnly;
	
	private class L extends Logger {
		private TextAreaLogHandler lh = new TextAreaLogHandler();

		public L(Level level) {
			super("",null);
			setUseParentHandlers(false);
			setLevel(level);
		}
		public void log(LogRecord record) {
			lh.publish(record);
		}
	}
	
	public synchronized boolean isDone() {
		return this.done;
	}
	
	public void setDone() {
		this.done = true;
		bc.setEnabled(false);
		bu.setEnabled(true);
	}
	
	private class TextAreaLogHandler extends Handler {
		private final static int MAX_LINES = 10000;
		private final StringBuilder toAppend = new StringBuilder();
		private Calendar cal = Calendar.getInstance();
	   	private StringBuilder sb = new StringBuilder(200);
		
		private Runnable appender = new Runnable() {

			public void run() {
				synchronized (TextAreaLogHandler.this) {
					if (toAppend.length() > 0) {
						ta.append(toAppend.toString());
						toAppend.setLength(0);
						int l = ta.getLineCount();
						if (l > MAX_LINES) {
							l -= MAX_LINES;
							try {
								int o = ta.getLineStartOffset(l);
								ta.getDocument().remove(0, o);
							} catch (BadLocationException e) {
								e.printStackTrace();
							}
						}
					}
				}
			}

		};

		public synchronized void publish(LogRecord record) {
			sb.setLength(0);
			if (!textOnly) {
		    	// Minimize memory allocations here.
		    	cal.setTime(new Date(record.getMillis()));
		    	int h = cal.get(Calendar.MONTH)+1;
		    	if (h<10)
		    	    sb.append('0');
		    	sb.append(h);
		    	sb.append("/");
		    	h = cal.get(Calendar.DAY_OF_MONTH);
		    	if (h<10)
		    	    sb.append('0');
		    	sb.append(h);
		    	sb.append(" ");
		    	h = cal.get(Calendar.HOUR_OF_DAY);
		    	if (h<10)
		    	    sb.append('0');
		    	sb.append(h);
			    sb.append(':');
		    	h = cal.get(Calendar.MINUTE);
		    	if (h<10)
		    	    sb.append('0');
		    	sb.append(h);
			    sb.append(':');
		    	h = cal.get(Calendar.SECOND);
		    	if (h<10)
		    	    sb.append('0');
		    	sb.append(h);
			    sb.append(' ');
			    sb.append('[');
		    	sb.append(record.getLevel().intValue());
			    sb.append(']');
			    sb.append(": ");
			}
	    	sb.append(record.getMessage());
	    	if (record.getThrown() != null) {
	    	    try {
	    	        StringWriter sw = new StringWriter();
	    	        PrintWriter pw = new PrintWriter(sw);
	    	        record.getThrown().printStackTrace(pw);
	    	        pw.close();
	    	        sb.append("\n");
	    	        sb.append(sw.toString());
	    	    } catch (Exception ex) {
	    	    }
	    	}
	        sb.append("\n");
			toAppend.append(sb.toString());
			SwingUtilities.invokeLater(appender);
		}

		public void flush() {
		}

		public void close() throws SecurityException {
		}
	}
	
	public ProgressLogger() {}
	
	public void run(Component owner, String title, Level level, final Runnable task) {
		Component f = owner;
		while ((f!=null) && (!(f instanceof Frame))) f=f.getParent();
		if (f==null)
			throw new IllegalStateException("Failed to identify frame!");
		this.di = new JDialog((Frame)f,title,true);
		
		JPanel mp = new JPanel();
		this.di.add(mp);
		BorderLayout bl = new BorderLayout();
		mp.setLayout(bl);
		mp.setBorder(BorderFactory.createEmptyBorder(5,10,5,10));

		// center log
		this.ta = new JTextArea();
		this.ta.setEditable(false);
		this.l=new L(level);
		mp.add(BorderLayout.CENTER,new JScrollPane(this.ta));
		
		// bottom button
		JPanel bp = new JPanel();
		mp.add(BorderLayout.SOUTH,bp);
		
		GridBagLayout grid = new GridBagLayout();
		bp.setLayout(grid);
		bp.setBorder(BorderFactory.createEmptyBorder(0,0,5,0));

		GridBagConstraints c= new GridBagConstraints();
		c.fill = GridBagConstraints.BOTH;
		c.insets = new Insets(5,2,5,2);

		c.weightx=0;
		c.gridwidth=1;
		this.bu = new JButton("Ok");
		this.bu.setEnabled(false);
		this.bu.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				di.setVisible(false);
			}
		});
		grid.setConstraints(this.bu,c);
		bp.add(this.bu);

		c.weightx=0;
		c.gridwidth=1;
		this.bc = new JButton("Cancel");
		this.bc.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				setDone();
			}
		});
		grid.setConstraints(this.bc,c);
		bp.add(this.bc);

		c.weightx=1;
		c.gridwidth=GridBagConstraints.REMAINDER;
		JLabel jl = new JLabel();
		grid.setConstraints(jl,c);
		bp.add(jl);

		// start the work
		ApplicationThreadPool.instance().execute(false,new Runnable() {
			public void run() {
				Cursor cu = di.getCursor();
				try {
					di.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
					task.run();
				} finally {
					setDone();
					di.setCursor(cu);
				}
			}
		});
		di.setPreferredSize(new Dimension(500,300));
		di.pack();
        di.setLocationRelativeTo(owner);

        // esc to cancel and close
        di.getRootPane().registerKeyboardAction(
        		new ActionListener(){
					public void actionPerformed(ActionEvent e) {
						if (ProgressLogger.this.isDone()) {
							di.setVisible(false);
						} else {
							ProgressLogger.this.setDone();
						}
					}
				}, 
        		KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0),
        		JComponent.WHEN_IN_FOCUSED_WINDOW
        );

        
        di.setVisible(true);
	}

	public Logger getLogger() {
		return this.l;
	}
	
	public void setTextOnly(boolean textOnly) {
		this.textOnly = textOnly;
	}
}
