/*
 * z2-Environment
 * 
 * Copyright(c) ZFabrik Software GmbH & Co. KG
 * 
 * contact@zfabrik.de
 * 
 * http://www.z2-environment.eu
 */
package com.zfabrik.impl.dev.transport.gui;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.regex.PatternSyntaxException;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.RowFilter;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableRowSorter;

import com.zfabrik.dev.transport.ProjectDelta;

public class ProjectsList implements ActionListener {
	private static final String AC_MARK_NONE = "NO";
	private static final String AC_MARK_ALL = "AL";
	private static final String AC_FILTER = "FI";
	private static final String AC_UNFILTER = "UF";
	private JPanel sw_component;
	private JTable sw_list;
	private TableRowSorter<ProjectsTableModel> sorter;
	private List<ProjectItem> projects = new ArrayList<ProjectItem>();
	private JTextField sw_filter;
	private ActionListener parent;

	public ProjectsList(ActionListener parent) {
		this.parent = parent;
		this.sw_component = new JPanel();

		BorderLayout bl = new BorderLayout();
		this.sw_component.setLayout(bl);
		this.sw_component.setBorder(BorderFactory.createCompoundBorder(BorderFactory.createTitledBorder("Candidate Projects"), BorderFactory
				.createEmptyBorder(5, 5, 5, 5)));

		// mark bar

		JPanel buttons = new JPanel();
		this.sw_component.add(BorderLayout.NORTH, buttons);

		GridBagLayout grid = new GridBagLayout();
		GridBagConstraints c = new GridBagConstraints();
		c.fill = GridBagConstraints.BOTH;
		c.insets = new Insets(5, 2, 5, 2);
		buttons.setLayout(grid);

		c.gridwidth = 1;
		c.weightx = 0;
		JButton bu = new JButton("compute");
		bu.setActionCommand(TransportTool.AC_COMPDELTA);
		bu.addActionListener(this);
		_addWithConstraints(buttons, grid, c, bu);

		c.gridwidth = 1;
		c.weightx = 1;
		this.sw_filter = new JTextField();
		_addWithConstraints(buttons, grid, c, this.sw_filter);

		c.gridwidth = 1;
		c.weightx = 0;
		bu = new JButton("filter");
		bu.setActionCommand(AC_FILTER);
		bu.addActionListener(this);
		_addWithConstraints(buttons, grid, c, bu);

		c.gridwidth = 1;
		c.weightx = 0;
		bu = new JButton("unfilter");
		bu.setActionCommand(AC_UNFILTER);
		bu.addActionListener(this);
		_addWithConstraints(buttons, grid, c, bu);

		c.gridwidth = 1;
		c.weightx = 0;
		bu = new JButton("mark all");
		bu.addActionListener(this);
		bu.setActionCommand(AC_MARK_ALL);
		bu.addActionListener(this);
		_addWithConstraints(buttons, grid, c, bu);

		c.gridwidth = 1;
		c.weightx = 0;
		bu = new JButton("mark none");
		bu.setActionCommand(AC_MARK_NONE);
		bu.addActionListener(this);
		_addWithConstraints(buttons, grid, c, bu);

		c.gridwidth = 1;
		c.weightx = 0;
		bu = new JButton("summary");
		bu.setToolTipText("Create a log summary for all marked projects");
		bu.setActionCommand(TransportTool.AC_SUMMARY);
		bu.addActionListener(this);
		_addWithConstraints(buttons, grid, c, bu);

		c.weightx = 0.3;
		c.gridwidth = GridBagConstraints.REMAINDER;
		_addWithConstraints(buttons, grid, c, new JLabel());

		ProjectsTableModel m = new ProjectsTableModel();
		this.sw_list = new JTable(m);
		
		JScrollPane sp = new JScrollPane(this.sw_list);
		this.sw_component.add(BorderLayout.CENTER, sp);

		this.sw_list.setFillsViewportHeight(true);
		this.sw_list.getSelectionModel().setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		// sorting & filtering
		this.sorter = new TableRowSorter<ProjectsTableModel>(m);
		this.sw_list.setRowSorter(this.sorter);

	}

	private void _addWithConstraints(JPanel p, GridBagLayout grid, GridBagConstraints c, Component comp) {
		grid.setConstraints(comp, c);
		p.add(comp);
	}

	private static class ProjectItem {
		private static final String[] COLS = new String[] { "apply", "name", "type" };
		boolean marked;
		ProjectDelta pd;

		public ProjectItem(ProjectDelta pd) {
			this.pd = pd;
		}

		public Object getColumn(int columnIndex) {
			switch (columnIndex) {
			case 0:
				return marked;
			case 1:
				return pd.getName();
			case 2:
				return Character.toString(pd.getType());
			}
			return null;
		}

		public static Class<?> getColumnClass(int columnIndex) {
			switch (columnIndex) {
			case 0:
				return Boolean.class;
			case 1:
				return String.class;
			case 2:
				return String.class;
			}
			return null;
		}

		public static int getColumnCount() {
			return 3;
		}

		public static String getColumnName(int column) {
			return COLS[column];
		}
		
		

	}

	@SuppressWarnings("serial")
	private class ProjectsTableModel extends AbstractTableModel {
		public int getColumnCount() {
			return ProjectItem.getColumnCount();
		}
		
		public int getRowCount() {
			return projects.size();
		}

		public Object getValueAt(int rowIndex, int columnIndex) {
			return projects.get(rowIndex).getColumn(columnIndex);
		}

		public Class<?> getColumnClass(int columnIndex) {
			return ProjectItem.getColumnClass(columnIndex);
		}

		public String getColumnName(int column) {
			return ProjectItem.getColumnName(column);
		}

		public void setValueAt(Object value, int rowIndex, int columnIndex) {
			if (columnIndex == 0) {
				ProjectItem pi = projects.get(rowIndex);
				pi.marked = ((Boolean) value).booleanValue();
			}
		}

		public boolean isCellEditable(int rowIndex, int columnIndex) {
			return (columnIndex == 0);
		}
	}

	public JComponent getComponent() {
		return this.sw_component;
	}

	public void actionPerformed(ActionEvent e) {
		String actionCommand = e.getActionCommand();
		if (TransportTool.AC_SUMMARY.equals(actionCommand)) {
			this.parent.actionPerformed(e);
		} else
		if (TransportTool.AC_COMPDELTA.equals(actionCommand)) {
			this.parent.actionPerformed(e);
		} else if ((AC_MARK_NONE.equals(actionCommand)) || (AC_MARK_ALL.equals(actionCommand))) {
			boolean mark = AC_MARK_ALL.equals(actionCommand);
			int l = this.sorter.getViewRowCount();
			for (int i = 0; i < l; i++) {
				this.projects.get(this.sorter.convertRowIndexToModel(i)).marked = mark;
			}
			((ProjectsTableModel) this.sw_list.getModel()).fireTableDataChanged();
			if (mark) {
				TransportTool.setStatus("All candidates marked.");
			} else {
				TransportTool.setStatus("All marks removed.");
			}
		} else if (AC_UNFILTER.equals(actionCommand)) {
			this.sw_filter.setText("");
			this.sorter.setRowFilter(null);
			TransportTool.setStatus("Filter condition cleared.");
		} else if (AC_FILTER.equals(actionCommand)) {
			String f = this.sw_filter.getText().trim();
			if (f.length() == 0) {
				this.sorter.setRowFilter(null);
				TransportTool.setStatus("Filter condition cleared.");
			} else {
				try {
					this.sorter.setRowFilter(RowFilter.regexFilter(f));
					TransportTool.setStatus("Filter condition set to \"" + f + "\".");
				} catch (PatternSyntaxException ex) {
					TransportTool.error(this.sw_component, "Bad filter expression (use regex)", ex);
				}
			}
		}
	}

	public void setProjectsList(Collection<ProjectDelta> pds) {
		this.projects = new ArrayList<ProjectItem>(pds.size());
		for (ProjectDelta pd : pds) {
			this.projects.add(new ProjectItem(pd));
		}
		((ProjectsTableModel) this.sw_list.getModel()).fireTableDataChanged();
	}

	public Collection<ProjectDelta> getDelta() {
		Set<ProjectDelta> r = new HashSet<ProjectDelta>();
		for (ProjectItem pi : this.projects) {
			if (pi.marked) {
				r.add(pi.pd);
			}
		}
		return r;
	}

    public boolean isAllMarked() {
        for (ProjectItem p : this.projects) {
            if (!p.marked) {
                return false;
            }
        }
        return true;
    }

	// private final static Logger logger =
	// Logger.getLogger(PackageList.class.getName());
}
