/*
 * z2-Environment
 * 
 * Copyright(c) ZFabrik Software GmbH & Co. KG
 * 
 * contact@zfabrik.de
 * 
 * http://www.z2-environment.eu
 */
package com.zfabrik.impl.dev.transport.gui;

import java.awt.BorderLayout;
import java.awt.Cursor;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.IOException;
import java.util.Arrays;
import java.util.Collection;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

import org.tmatesoft.svn.core.SVNException;
import org.tmatesoft.svn.core.SVNURL;
import org.tmatesoft.svn.core.auth.BasicAuthenticationManager;
import org.tmatesoft.svn.core.internal.io.dav.DAVRepositoryFactory;
import org.tmatesoft.svn.core.internal.io.fs.FSRepositoryFactory;
import org.tmatesoft.svn.core.internal.io.svn.SVNRepositoryFactoryImpl;
import org.tmatesoft.svn.core.io.SVNRepository;
import org.tmatesoft.svn.core.io.SVNRepositoryFactory;
import org.tmatesoft.svn.core.wc.SVNRevision;

import com.zfabrik.components.IComponentDescriptor;
import com.zfabrik.components.IComponentsManager;
import com.zfabrik.dev.transport.ICancellation;
import com.zfabrik.dev.transport.ProjectDelta;
import com.zfabrik.dev.transport.Transport;
import com.zfabrik.swing.ZFrame;
import com.zfabrik.util.expression.X;
import com.zfabrik.util.fs.FileUtils;

public class TransportTool implements Runnable, ActionListener {

    public final static String AC_PICK_STARTREV = "PSR";
    public final static String AC_PICK_ENDREV = "PER";
    public final static String AC_PICK_TARGETREV = "PTR";
    public final static String AC_COMPUTE_PKGS = "CPL";
    public static final String AC_RUN  = "RU";
    public static final String AC_COMPDELTA = "CDE";
    public static final String AC_SUMMARY = "SUM";
    private static final String L_FINE = "fine";
    private static final String L_INFO = "info";
    private static final String AC_REFRDESTS = "RFD";

    private JComponent component;

    private DestinationChooser sw_sourceChooser, sw_targetChooser;
    private String[] dests = new String[0];
    private ProjectsList sw_list;
    private RevisionChooser sw_revisionChooser;
    private WorkSpaceChooser sw_workspaceChooser;
    private RevPicker di_rp = new RevPicker();
    private JCheckBox sw_removeUnkown;
    private JComboBox sw_logLevel;
    private JCheckBox sw_onlyUsersProject;

    private static ZFrame zf;

    /*
     * The MAIN method (executed in swing worker)
     */
    public synchronized void run() {
        JPanel mp = new JPanel();
        BorderLayout bo = new BorderLayout();
        mp.setLayout(bo);

        // upper part
        JPanel north = new JPanel();
        mp.add(BorderLayout.NORTH, north);

        BoxLayout bl = new BoxLayout(north, BoxLayout.Y_AXIS);
        north.setLayout(bl);

        // source and target repo chooser
        JPanel st = new JPanel();
        north.add(st);
        bl = new BoxLayout(st, BoxLayout.X_AXIS);
        st.setLayout(bl);

        JPanel sc = new JPanel();
        st.add(sc);
        sc.setBorder(BorderFactory.createCompoundBorder(BorderFactory.createTitledBorder("Source Repository"), BorderFactory.createEmptyBorder(5,
                5, 5, 5)));
        this.sw_sourceChooser = new DestinationChooser(this, dests);
        sc.add(this.sw_sourceChooser);

        sc = new JPanel();
        st.add(sc);

        sc.setBorder(BorderFactory.createCompoundBorder(BorderFactory.createTitledBorder("Target Repository"), BorderFactory.createEmptyBorder(5,
                5, 5, 5)));
        this.sw_targetChooser = new DestinationChooser(this, dests);
        sc.add(this.sw_targetChooser);

        // revision and settings
        st = new JPanel();
        north.add(st);

        bl = new BoxLayout(st, BoxLayout.X_AXIS);
        st.setLayout(bl);

        this.sw_revisionChooser = new RevisionChooser(this);
        st.add(this.sw_revisionChooser);

        sc = new JPanel();
        sc.setLayout(new BoxLayout(sc,BoxLayout.X_AXIS));

        st.add(sc);
        sc.setBorder(BorderFactory.createCompoundBorder(
                BorderFactory.createTitledBorder("Settings"),
                BorderFactory.createEmptyBorder(5,5,5,5)));

        JPanel vLayout = new JPanel();
        vLayout.setLayout(new BoxLayout(vLayout,BoxLayout.Y_AXIS));
        sc.add(vLayout);

        this.sw_removeUnkown = new JCheckBox("Filter unknown M/D proj ",true);
        vLayout.add(this.sw_removeUnkown);

        this.sw_onlyUsersProject = new JCheckBox("Filter on User's projects ", false);
        this.sw_onlyUsersProject.setToolTipText("Candidate Projects are limited to projects that were touched by the (source) user in the given revision range");
        vLayout.add(this.sw_onlyUsersProject);


        JButton bu = new JButton("R. Dests");
        sc.add(bu);
        bu.setActionCommand(AC_REFRDESTS);
        bu.addActionListener(this);

        JPanel gr = new JPanel();
        sc.add(gr);
        gr.setLayout(new BoxLayout(gr,BoxLayout.X_AXIS));
        gr.add(new JLabel(" Logging Level: "));
        this.sw_logLevel = new JComboBox(new String[]{L_INFO,L_FINE});
        gr.add(this.sw_logLevel);

        // center part
        this.sw_list = new ProjectsList(this);
        mp.add(BorderLayout.CENTER, this.sw_list.getComponent());

        this.sw_workspaceChooser = new WorkSpaceChooser(this);
        mp.add(BorderLayout.SOUTH, this.sw_workspaceChooser);

        this.component = mp;

        zf = new ZFrame("Transport Tool");
        zf.getContentPanel().add(this.component);
        zf.pack();
        zf.setVisible(true);
        _readDests();
    }

    private void _readDests() {
        setStatus("Reading destinations...");
        try {
            Collection<String> dc = IComponentsManager.INSTANCE.findComponents(
        		X.var(IComponentDescriptor.COMPONENT_TYPE)
        		.eq(
    				X.val("com.zfabrik.svncr")
				)
			);
            this.dests = dc.toArray(new String[dc.size()]);
            Arrays.sort(this.dests);
            this.sw_sourceChooser.setDestinations(this.dests);
            this.sw_targetChooser.setDestinations(this.dests);
            setStatus("Destinations read.");
        } catch (IOException e) {
            throw new IllegalStateException("Failed to query destinations");
        }
    }

    public void actionPerformed(ActionEvent e) {
        String actionCommand = e.getActionCommand();
        if (AC_PICK_STARTREV.equals(actionCommand)) {
            if (di_rp.run(this.sw_sourceChooser.getDestination(), this.component)) {
                this.sw_revisionChooser.setStartRev(di_rp.getRev());
                setStatus("Chose source start revision "+di_rp.getRev()+".");
            }
        } else if (AC_PICK_ENDREV.equals(actionCommand)) {
            if (di_rp.run(this.sw_sourceChooser.getDestination(), this.component)) {
                this.sw_revisionChooser.setEndRev(di_rp.getRev());
                setStatus("Chose source end revision "+di_rp.getRev()+".");
            }
        } else if (AC_PICK_TARGETREV.equals(actionCommand)) {
            if (di_rp.run(this.sw_targetChooser.getDestination(), this.component)) {
                this.sw_revisionChooser.setTargetRev(di_rp.getRev());
                setStatus("Chose target revision "+di_rp.getRev()+".");
            }
        } else if (AC_SUMMARY.equals(actionCommand)) {
            _computeSummary();
        } else if (AC_COMPDELTA.equals(actionCommand)) {
            _computeDelta();
        } else if (AC_RUN.equals(actionCommand)) {
            _generateWorkspace();
        } else if (AC_REFRDESTS.equals(actionCommand)) {
            _readDests();
        }
    }

    private void _generateWorkspace() {
        // do it
        final File workspace = this.sw_workspaceChooser.getWorkspaceRoot();

        if (workspace==null) {
            error(this.component,"Please specify workspace target folder", null);
            return;
        }
        if (workspace.exists()) {
            if (JOptionPane.showConfirmDialog(this.component, 
                    "Target folder\n"+workspace+"\nexists. It will be deleted during generation.", "Workspace exists. Continue?",
                    JOptionPane.OK_CANCEL_OPTION)!=JOptionPane.OK_OPTION) return;
        }
        // logger dialog
        final ProgressLogger pl = new ProgressLogger();
        Level l = Level.INFO;
        if (L_FINE.equals(this.sw_logLevel.getSelectedItem())) {
            l = Level.FINE;
        }
        setStatus("Workspace generation in progress...");
        pl.run(this.component,"Workspace Generation ("+workspace+")",l,
                new Runnable() {
            public void run() {
                Logger l = pl.getLogger();
                try {
                    // if sw_onlyUsersProject is selected, the delta workspace is not generated to the end-revision, but to the maximum revision of each project. 
                    // This is stored for each DeltaProject object individually!  
                    SVNRevision erev = TransportTool.this.sw_onlyUsersProject.isSelected()? null : SVNRevision.parse(sw_revisionChooser.getEndRev());
                    SVNRevision trev = SVNRevision.parse(sw_revisionChooser.getTargetRev());

                    SVNRepository sr = getRepository(sw_sourceChooser.getDestination());
                    SVNRepository tr = getRepository(sw_targetChooser.getDestination());

                    FileUtils.delete(workspace);

                    Transport.createDeltaWorkSpace(sr, erev, tr, trev, sw_list.getDelta(), workspace, l, new ICancellation() {
                        public boolean isCancelled() {return pl.isDone();}
                    });
                    setStatus("Workspace generation completed.");
                } catch (Exception e) {
                    l.log(Level.SEVERE,"Workspace generation failed",e);
                    setStatus("Workspace generation failed.");
                }
            }
        });
    }
    
    private void _computeSummary() {
        final Collection<ProjectDelta> delta = this.sw_list.getDelta();
        if (delta.size()==0) {
            error(this.component, "Please mark projects to create a summary for.", null);
        } else {
            final SVNRepository sr = getRepository(this.sw_sourceChooser.getDestination());
            final ProgressLogger pl = new ProgressLogger();
            pl.setTextOnly(true);
            setStatus("Computing log summary...");
            pl.run(this.component,"Log Summary",Level.INFO,
                    new Runnable() {
                public void run() {
                    final Logger l = pl.getLogger();
                    try {
                        Transport.writeSummary(
                            delta, 
                            sr, 
                            SVNRevision.parse(sw_revisionChooser.getStartRev()),
                            SVNRevision.parse(sw_revisionChooser.getEndRev()),
                            sw_list.isAllMarked(), 
                            l
                        );
                        setStatus("Log summary completed.");
                    } catch (Exception e) {
                        l.log(Level.SEVERE,"Failed to compute log summary",e);
                        setStatus("Log summary computation failed.");
                    }
                }
            });
        }
    }

    private void _computeDelta() {
        Cursor c = this.component.getCursor();
        setStatus("Determining transport candidates...");
        try {
            this.component.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
            SVNRevision srev = SVNRevision.parse(this.sw_revisionChooser.getStartRev());
            SVNRevision erev = SVNRevision.parse(this.sw_revisionChooser.getEndRev());
            SVNRevision trev = SVNRevision.parse(this.sw_revisionChooser.getTargetRev());

            SVNRepository sr = getRepository(this.sw_sourceChooser.getDestination());
            SVNRepository tr = getRepository(this.sw_targetChooser.getDestination());

            // get packages on target side so we can cleanly handle adds and simplify the filtering
            Collection<String> tps = Transport.getTargetProjects(tr, trev);

            String filterUser = this.sw_onlyUsersProject.isSelected()? this.sw_sourceChooser.getUserName() : null;

            // compute the delta set 
            Collection<ProjectDelta> pds = Transport.computeProjectDeltas(tps, sr, srev, erev, filterUser, null);

            if (this.sw_removeUnkown.isSelected()) {
                Transport.removeNonApplicableProjects(tps, pds);
            }

            this.sw_list.setProjectsList(pds);

            setStatus("Transport candidates determination completed.");
        } catch (Exception e) {
            error(this.component, "Failed to compute transport candidates", e);
            logger.log(Level.WARNING,"Failed to compute transport candidates", e);
        } finally {
            this.component.setCursor(c);
        }
    }

    public void dispose() {
        this.di_rp.cancel();
    }

    public static void error(JComponent owner, String message, Throwable t) {
        JOptionPane.showMessageDialog(owner, message + (t != null ? " (" + t + ")" : ""), "Error", JOptionPane.ERROR_MESSAGE);
        setStatus("Error: "+message + (t != null ? " (" + t + ")" : ""));
    }

    public static void setStatus(String text) {
        if (zf!=null) {
            zf.status(text);
        }
    }


	public static SVNRepository getRepository(Destination dest) {
		// take care of all the pre-reqs
        ClassLoader cl = Thread.currentThread().getContextClassLoader();
        Thread.currentThread().setContextClassLoader(null);
        SVNRepository repo = null;
        try {
            String proxyHost = System.getProperty("http.proxyHost",System.getProperty("proxyHost"));
            String proxyPort = System.getProperty("http.proxyPort",System.getProperty("proxyPort"));
            String url = dest.getUrl();
            if (url.indexOf("://")<0) {
                url = "file:///"+new File("").toURI().toString().substring("file:/".length())+url;
            }
            SVNURL svnurl = SVNURL.parseURIEncoded(url);
			if (url.startsWith("svn:")) {
                SVNRepositoryFactoryImpl.setup();
                repo = SVNRepositoryFactory.create(svnurl);
            } else if (url.startsWith("http:")) {
                DAVRepositoryFactory.setup();
                repo = DAVRepositoryFactory.create(svnurl);
            } else if (url.startsWith("https:")) {
                DAVRepositoryFactory.setup();
                repo = DAVRepositoryFactory.create(svnurl);
            } else if (url.startsWith("file:")) {
                FSRepositoryFactory.setup();
                repo = FSRepositoryFactory.create(svnurl);
            } else
                throw new RuntimeException("SVNKit SVN access: don't know how to handle \""+url+"\"");

			// we configure an auth manager. The default uses the current OS user's settings. We definitely do not want that.
			//
	        BasicAuthenticationManager authManager = new BasicAuthenticationManager(dest.getUser(),dest.getPassword());

	        // configure proxy use for SVNKit
	        if (proxyHost!=null && (proxyHost=proxyHost.trim()).length()>0 && proxyPort!=null && (proxyPort=proxyPort.trim()).length()>0) {
                int pp;
                try {
                    pp = Integer.parseInt(proxyPort.trim());
                } catch (NumberFormatException nfe) {
                    throw new RuntimeException("Failed to configure SVNKit proxy settings - invalid port number \""+proxyPort+"\"",nfe);
                }
                authManager.setProxy(proxyHost,pp, System.getProperty("proxyUser"), System.getProperty("proxyPassword"));
	        }

	        repo.setAuthenticationManager(authManager);
		        
        } catch (SVNException svne) {
        	throw new RuntimeException(svne);
        } finally {
        	Thread.currentThread().setContextClassLoader(cl);
        }
        return repo;
    }

    private final static Logger logger = Logger.getLogger(TransportTool.class.getName());

}