/*
 * z2-Environment
 * 
 * Copyright(c) ZFabrik Software GmbH & Co. KG
 * 
 * contact@zfabrik.de
 * 
 * http://www.z2-environment.eu
 */
package com.zfabrik.impl.cluster;
import java.util.logging.Logger;

import com.zfabrik.components.IDependencyComponent;
import com.zfabrik.resources.ResourceBusyException;
import com.zfabrik.resources.provider.Resource;
import com.zfabrik.sync.SynchronizationRunner;
import com.zfabrik.util.runtime.Foundation;


public class ClusterSynchronizationTrigger extends Resource implements IDependencyComponent {
	private String key = ClusterSynchronizationTrigger.class.getName()+"/pending";
	
	public ClusterSynchronizationTrigger() {
	}
	
	public <T> T as(Class<T> clz) {
		if (IDependencyComponent.class.equals(clz))
			return clz.cast(this);
		return null;
	}

	public void prepare() {
		synchronized (this) {
			if (this._isPendingSync()) {
				this._setPendingSync(false);
				logger.info("Triggering pending synchronization");
				new Thread(new Runnable() {
					public void run() {
						new SynchronizationRunner(SynchronizationRunner.INVALIDATE_AND_VERIFY).execute(false);
					}
				}).start();
			}
		}
	}
	
	private boolean _isPendingSync() {
		Boolean b = (Boolean) Foundation.getProperties().get(key);
		return (b!=null) && b;
	}

	private void _setPendingSync(boolean b) {
		Foundation.getProperties().put(key, b);
	}

	public synchronized void invalidate() throws ResourceBusyException {
		// a synchronization gets triggered upon invalidation of this trigger
		this._setPendingSync(true);
	}
	
	private final static Logger logger = Logger.getLogger(ClusterSynchronizationTrigger.class.getName());
}
