/*
 * z2-Environment
 * 
 * Copyright(c) ZFabrik Software GmbH & Co. KG
 * 
 * contact@zfabrik.de
 * 
 * http://www.z2-environment.eu
 */
package com.zfabrik.impl.cluster.dao;

import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.util.Collection;
import java.util.LinkedList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.sql.DataSource;

import com.zfabrik.work.IWorkResource;
import com.zfabrik.work.WorkException;
import com.zfabrik.work.WorkUnit;

public class InvalidationsDAO {

	
	private DataSource ds; 
	private InvalidationsImpl dialectImpl;
	
	
	public InvalidationsDAO(DataSource ds, String tablePrefix)  throws Exception  {
		if (tablePrefix==null)
			tablePrefix = "Z2_CLUSTER_";
		this.ds = ds;

		Connection c = this.ds.getConnection();
		// load the dialect
		DatabaseMetaData md = c.getMetaData();
		String dialect = md.getDatabaseProductName().replace(" ","_");
		String clzName = this.getClass().getName();
		if (logger.isLoggable(Level.FINER)) {
			logger.finer("Using dialect "+dialect);
		}
		clzName = clzName.substring(0,clzName.lastIndexOf('.'))+".InvalidationsDAOFor"+dialect;
		try {
			this.dialectImpl = (InvalidationsImpl) Class.forName(clzName).newInstance();
			this.dialectImpl.init(c,tablePrefix);
		} catch (Exception e) {
			throw new IllegalStateException("Failed to load and initialize implementation ("+clzName+") for dialect \""+dialect+"\"",e);
		}
	}
	
	
	public List<String> pull(int max) throws Exception {
		Connection c = this.ds.getConnection();
		return this.dialectImpl.pull(c,max);
	}
	
	public void post(Collection<String> invs) throws Exception {
		InvDU du = (InvDU) WorkUnit.getCurrent().getResource(InvDU.class.getName());
		if (du==null) {
			du = new InvDU(this.ds,this.dialectImpl);
			WorkUnit.getCurrent().bindResource(InvDU.class.getName(), du);
		}
		du.getInvs().addAll(invs);
	}
	
	
	// --------- workunit to collect invalidations until a prepare and commit ------- //
	private static class InvDU implements IWorkResource {
		private List<String> invs = new LinkedList<String>();
		
		private DataSource ds;
		private InvalidationsImpl invi;
		
		public InvDU(DataSource ds, InvalidationsImpl invi) {
			super();
			this.ds = ds;
			this.invi = invi;
		}

		public List<String> getInvs() {
			return invs;
		}
		
		public void afterCompletion(boolean rollback) {}
		
		public void beforeCompletion(boolean rollback) throws WorkException {
			if (!rollback) {
				try {
					if (this.invs.size()>0) {
						this.invi.post(ds.getConnection(),this.invs);
						this.invs.clear();
					}
				} catch (Exception e) {
					throw new WorkException("Failed to post pending invalidations",e);
				}
			}
		}

		public void close() throws WorkException {
			
		}

		public void begin() throws WorkException {}
		public void commit() throws WorkException {}
		public void rollback() throws WorkException {}
	}

	
	private final static Logger logger = Logger.getLogger(InvalidationsDAO.class.getName()); 
}
