/*
 * z2-Environment
 * 
 * Copyright(c) ZFabrik Software GmbH & Co. KG
 * 
 * contact@zfabrik.de
 * 
 * http://www.z2-environment.eu
 */
package com.zfabrik.impl.workers.home;

import java.util.logging.Logger;

import com.zfabrik.components.IComponentsLookup;
import com.zfabrik.components.IDependencyComponent;
import com.zfabrik.resources.IResourceHandle;
import com.zfabrik.resources.ResourceBusyException;
import com.zfabrik.resources.provider.Resource;
import com.zfabrik.util.runtime.Foundation;
import com.zfabrik.workers.home.IHomeLayout;
import com.zfabrik.workers.home.IWorkersManager;

/**
 * The workers manager essentially manages a list of active workers and initializes
 * the home layout. 
 * 
 * @author hb
 *
 */
public class WorkersManager extends Resource implements IDependencyComponent,IWorkersManager {

	private IResourceHandle layout;
	private boolean inited;

	public <T> T as(Class<T> clz) {
		if (IDependencyComponent.class.equals(clz)) {
			return clz.cast(this);
		}
		if (IWorkersManager.class.equals(clz)) {
			return clz.cast(this);
		}
		return null;
	}
	
	
	
	@Override
	public void invalidate()
			throws ResourceBusyException {
		stop(); 
	}

	private void stop() {
		synchronized (this) {
			this.inited = false;
			this.layout = null;
		}
	}

	/**
	 * TODO: concurrent startup
	 */
	public void prepare() {
		synchronized (this) {
			// load this manager only if this is the home!
			if (Foundation.isWorker()) {
				throw new IllegalStateException(
						"Cannot run the workers manager on a worker process!");
			}
			// get the home layout requested
			IHomeLayout h = getActiveLayout();
			if (h!=null) {
				h.start();
			}
		}
	}

	public IHomeLayout getActiveLayout() {
		synchronized (this) {
			if (!this.inited) {
				String layoutProp = Foundation.getProperties().getProperty(
						Foundation.HOME_LAYOUT_COMPONENT);
				if (layoutProp != null) {
					this.layout = IComponentsLookup.INSTANCE.lookup(layoutProp,IResourceHandle.class); 
					handle().addDependency(this.layout);
				} else {
					logger.fine("Note: No home layout configured.");
				}
				this.inited = true;
			}
			if (this.layout!=null) {
				IHomeLayout l = this.layout.as(IHomeLayout.class);
				if (l==null) {
					throw new IllegalStateException("Failed to retrieve home layout "+this.layout.getResourceInfo().getName());
				}
				return l;
			} 
			return null;
		}
	}
	
	public Resource createWorkerResource(IResourceHandle dependant, String name) {
		if (dependant!=null) {
			dependant.addDependency(handle());
		}
		return new WorkerProcessResource(name);
	}
	
	 private final static Logger logger = Logger.getLogger(WorkersManager.class.getName());
}
